var baseFlatten = require('./_baseFlatten'),
    map = require('./map');

/**
 * Creates an array of flattened values by running each element in `collection`
 * through `iteratee` and concating its result to the other mapped values.
 * The iteratee is invoked with three arguments: (value, index|key, collection).
 *
 * @static
 * @memberOf _
 * @category Collection
 * @param {Array|Object} collection The collection to iterate over.
 * @param {Function|Object|string} [iteratee=_.identity] The function invoked per iteration.
 * @returns {Array} Returns the new flattened array.
 * @example
 *
 * function duplicate(n) {
 *   return [n, n];
 * }
 *
 * _.flatMap([1, 2], duplicate);
 * // => [1, 1, 2, 2]
 */
function flatMap(collection, iteratee) {
  return baseFlatten(map(collection, iteratee));
}

module.exports = flatMap;
