var createWrapper = require('./_createWrapper');

/** Used to compose bitmasks for wrapper metadata. */
var FLIP_FLAG = 512;

/**
 * Creates a function that invokes `func` with arguments reversed.
 *
 * @static
 * @memberOf _
 * @category Function
 * @param {Function} func The function to flip arguments for.
 * @returns {Function} Returns the new function.
 * @example
 *
 * var flipped = _.flip(function() {
 *   return _.toArray(arguments);
 * });
 *
 * flipped('a', 'b', 'c', 'd');
 * // => ['d', 'c', 'b', 'a']
 */
function flip(func) {
  return createWrapper(func, FLIP_FLAG);
}

module.exports = flip;
